"""

"""
import datetime
import time


def sub_sample_1_1_now():
    return datetime.datetime.now()


def sub_sample_1_1_today():
    return datetime.date.today()


def sub_sample_1_1_sleep(seconds):
    datetime.time.sleep(seconds)


def sub_sample_1_1_time():
    return time.time()
