"""

"""
import time
from datetime import date, datetime


def sub_sample_2_1_now():
    return datetime.now()


def sub_sample_2_1_today():
    return date.today()


def sub_sample_2_1_sleep(seconds):
    time.sleep(seconds)


def sub_sample_2_1_time():
    return time.time()
